package com.bipt.intelligentapplicationorchestrationservice.gpu.cache;


import com.bipt.intelligentapplicationorchestrationservice.gpu.model.entity.GpuResource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.stereotype.Service;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Service
public class RedisCacheService {

    private final RedisTemplate<String, Object> redisTemplate;

    @Value("${cache.redis-key-prefix:gpu:}")
    private String keyPrefix;

    @Value("${cache.ttl-base:7200}")
    private int baseTTL;
    private final RedisSerializer<Object> valueSerializer;

//    @Autowired
//    public RedisCacheService(RedisTemplate<String, Object> redisTemplate) {
//        this.redisTemplate = redisTemplate;
//    }

    // 核心方法 ------------------------------------------------------------
    @Autowired
    public RedisCacheService(RedisTemplate<String, Object> redisTemplate) {
        this.redisTemplate = redisTemplate;
        this.valueSerializer = (RedisSerializer<Object>) redisTemplate.getValueSerializer();
    }

    /**
     * 批量写入GPU资源数据（带管道优化）
     * @param resources GPU资源列表
     */
    public void batchPut(List<GpuResource> resources) {
        redisTemplate.executePipelined((RedisCallback<Object>) connection -> {
            resources.forEach(resource -> {
                String key = buildKey(resource.getGPUId().toString());
                byte[] keyBytes = key.getBytes(StandardCharsets.UTF_8);
                byte[] valueBytes = valueSerializer.serialize(resource);
                connection.stringCommands().setEx(
                        keyBytes,
                        calculateTTL(),
                        valueBytes
                );
            });
            return null;
        });
    }

    /**
     * 设置单条缓存（含随机TTL防雪崩）
     * @param gpuId 资源ID
     * @param resource 资源对象
     */
    public void put(String gpuId, GpuResource resource) {
        String key = buildKey(gpuId);
        redisTemplate.opsForValue().set(
                key,
                resource,
                calculateTTL(),
                TimeUnit.SECONDS
        );
    }

    /**
     * 获取单个缓存项
     * @param gpuId 资源ID
     * @return 缓存对象或null
     */
    public GpuResource get(String gpuId) {
        return (GpuResource) redisTemplate.opsForValue().get(buildKey(gpuId));
    }

    /**
     * 删除指定缓存
     * @param gpuId 资源ID
     */
    public void delete(Long gpuId) {
        redisTemplate.delete(buildKey(gpuId.toString()));
    }

    // 辅助方法 ------------------------------------------------------------

    private String buildKey(String gpuId) {
        return keyPrefix + gpuId;
    }

    private long calculateTTL() {
        return baseTTL + (long)(Math.random() * 600); // 7200-7800秒随机值
    }

    /**
     * 批量删除缓存（事务处理）
     * @param gpuIds 资源ID列表
     */
    public void batchDelete(List<String> gpuIds) {
        redisTemplate.execute((RedisCallback<Object>) connection -> {
            connection.multi();
            gpuIds.forEach(id -> connection.del(buildKey(id).getBytes()));
            connection.exec();
            return null;
        });
    }

    /**
     * 缓存健康检查
     * @return 是否连通
     */
    public boolean healthCheck() {
        try {
            return "PONG".equals(redisTemplate.getConnectionFactory()
                    .getConnection().ping());
        } catch (Exception e) {
            return false;
        }
    }
}